/*
 ***********************************************************************************************
 *
 *  OSA cooperative RTOS for Microchip PIC-controllers: PIC10/12/16/18/24/dsPIC
 *
 *  OSA is distributed under BSD license (see license.txt)
 *
 *  URL:            http://wiki.pic24.ru/doku.php/en/osa/ref/intro
 *                  http://picosa.narod.ru
 *
 *----------------------------------------------------------------------------------------------
 *
 *  File:           osa_mikroc18.h
 *
 *  Compilers:      mikroC PRO 2.50 and higher
 *
 *  Programmer:     Timofeev Victor
 *                  osa@pic24.ru, testerplus@mail.ru
 *
 *  Description:    mikroC for PIC18 specific definitions
 *                  The specific of mikroC cmpiler is that defined in macro inline asm {} will
 *                  be igored and not included into code.
 *
 *  History:        21.01.2009
 *                  01.04.2009 -    All processor specific definitions moved from osa.h into
 *                                  this file
 * 
 *                  10.12.2009 -    Added empty macro OS_Task_Replace for code compablity with
 *                                  other compilers
 *                                  Added services OS_DI_INT and OS_RI_INT
 *
 ***********************************************************************************************
 */
#ifndef __OSAMIKROC18_H__
#define __OSAMIKROC18_H__



/************************************************************************************************
 *                                                                                              *
 *     Integer types definitions                                                                *
 *                                                                                              *
 ************************************************************************************************/

typedef unsigned char   __osa_uint8;
typedef unsigned int    __osa_uint16;
typedef unsigned long   __osa_uint32;
typedef unsigned char   __osa_bool;

typedef __osa_uint8     _OST_INT_TYPE;
#define _OST_INT_MASK    7
#define _OST_INT_SHIFT   3
#define _OST_INT_SIZE    8



/************************************************************************************************
 *                                                                                              *
 *     Location keywords                                                                        *
 *                                                                                              *
 ************************************************************************************************/

#define NEAR
#define bank0
#define bank1
#define bank2
#define bank3
#define __osa30_near
#define _OS_CUR_FLAGS_IN_OS_state        // Work with tasks state througth OS_state



/************************************************************************************************
 *                                                                                              *
 *     Processor specified definitions for system flags                                         *
 *     (see OST_SYSTEM_FLAGS and OST_TCB definition in file "OSA.h")                            *
 *                                                                                              *
 ************************************************************************************************/

#define _OST_SYSTEM_FLAGS_PROC_SPEC                                                             \
                                                                                                \
        /*5*/   _OST_INT_TYPE   bTimeout    : 1; /* Event waiting timeout                */     \
        /*6*/   _OST_INT_TYPE   bGIE_CTemp  : 1; /* Saved GIEH while in critical section */     \
        /*7*/   _OST_INT_TYPE   bGIEL_CTemp : 1; /* Saved GIEL while in critical section */     \

#define _OST_TCB_PROC_SPEC      // Empty macro for this processor


#define OST_TASK_POINTER   OS_TASKS_BANK OST_TCB *

/************************************************************************************************
 *                                                                                              *
 *     Registers definitions                                                                    *
 *                                                                                              *
 ************************************************************************************************/

extern volatile unsigned int _fsr;
extern volatile char         _indf;
extern volatile char         _postinc;
extern volatile char         _postdec;
extern volatile char         _preinc;

extern volatile unsigned char _fsr1l;

extern volatile char         _pcl;
extern volatile char         _pclath;
extern volatile char         _pclatu;
extern volatile char         _status;
extern volatile char         _tosl;
extern volatile char         _tosh;
extern volatile char         _tosu;
extern volatile char         _bsr;
extern volatile char         _wreg;
extern volatile char         _intcon;
extern volatile char         _rcon;
extern volatile char         _stkptr;

#define _gie                GIE_bit
#define _giel               GIEL_bit
#define _ipen               IPEN_bit



/************************************************************************************************
 *                                                                                              *
 *     Constants and types                                                                      *
 *                                                                                              *
 ************************************************************************************************/

#ifndef OS_SMSG_SIZE
#define OS_SMSG_SIZE    0   // We can not use sizeof() because it does not work with #ifdef directive
#endif

//------------------------------------------------------------------------------

#define _FSR_TYPE            unsigned int

//------------------------------------------------------------------------------

#if defined(_ROMSIZE) && (_ROMSIZE <= 0x10000)

    #define OST_CODE_POINTER        unsigned int
    #define OS_CODE_POINTER_SIZE    2

#else

    #define OST_CODE_POINTER        unsigned long
    #define OS_CODE_POINTER_SIZE    4

#endif





/************************************************************************************************
 *                                                                                              *
 *     Platform macros                                                                          *
 *                                                                                              *
 ************************************************************************************************/

#define _OS_POP18()             asm{ pop }


#if defined(_ROMSIZE) && (_ROMSIZE <= 0x4000)
#define _PIC18_ERRATA_NOP()
#else
#define _PIC18_ERRATA_NOP()     asm{    nop     }
#endif


#define OS_CLRWDT()             asm{ clrwdt }
#define OS_ClrWdt()             asm{ clrwdt }
#define OS_Clrwdt()             asm{ clrwdt }
#define OS_SLEEP()              asm{ sleep  }
#define OS_Sleep()              asm{ sleep  }








/************************************************************************************************
 *                                                                                              *
 *     Context switching macros                                                                 *
 *                                                                                              *
 ************************************************************************************************/


//------------------------------------------------------------------------------
// Save/restore PC macros
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
#if OS_CODE_POINTER_SIZE == 2
//------------------------------------------------------------------------------

    #define _OS_SetPC()                         \
        _OS_temp = _postinc;                    \
        _pclath =  _postinc;


    #define _OS_SavePC()                        \
        _preinc = _tosl;                        \
        _preinc = _tosh;                        \

//------------------------------------------------------------------------------
#else
//------------------------------------------------------------------------------

    #define _OS_SetPC()                         \
        _OS_temp = _postinc;                    \
        _pclath =  _postinc;                    \
        _pclatu =  _postinc;


    #define _OS_SavePC()                        \
        _preinc = _tosl;                        \
        _preinc = _tosh;                        \
        _preinc = _tosu;                        \

//------------------------------------------------------------------------------
#endif
//------------------------------------------------------------------------------


/*
 ********************************************************************************
 *                                                                              *
 *  _OS_JumpToTask()                                                            *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:        jump indirectly from kernel (OS_Sched) into task        *
 *                                                                              *
 ********************************************************************************
 */

void __OS_JumpToTask (void);

#define _OS_JumpToTask()                            \
    {                                               \
        __OS_JumpToTask();                          \
        _OS_SET_FSR_CUR_TASK();                     \
        _indf &= 0xD7;                              \
        _indf |= *((char*)&OS_state) & 0x28;        \
    }

#define _OS_SaveRetAddr()






/************************************************************************************************
 *                                                                                              *
 *     Return from task to OS_Sched macros                                                      *
 *                                                                                              *
 ************************************************************************************************/


/*
 ********************************************************************************
 *                                                                              *
 *   Switch control routines:                                                   *
 *                                                                              *
 *    _OS_RETURN_SAVE()                   - exit from task and save context     *
 *                                                                              *
 *    _OS_RETURN_NO_SAVE()                - exit withowt saving return address  *
 *                                                                              *
 *    _OS_CLEAR_READY()                   - exit with saving address and        *
 *                                          clearing bReady                     *
 *                                                                              *
 *    _OS_CLEAR_READY_SET_CANCONTINUE()   - exit without saving address and     *
 *                                          clearing bReady and bCanContinue    *
 *                                                                              *
 ********************************************************************************
 */


extern void OS_SchedRetPoint (void);
extern void OS_SchedRetPointNoSave (void);
extern void OS_ClearReady (void);
extern void OS_ClearReadySetCanContinue (void);

extern void  _OS_SET_FSR_CUR_TASK (void);


//------------------------------------------------------------------------------
#define _OS_RETURN_SAVE()                       \
    {                                           \
        OS_SchedRetPoint();                     \
        _PIC18_ERRATA_NOP();                    \
    }

//------------------------------------------------------------------------------
#define _OS_CLEAR_READY()                       \
    {                                           \
        OS_ClearReady();                        \
        _PIC18_ERRATA_NOP();                    \
    }
//------------------------------------------------------------------------------
#define _OS_CLEAR_READY_SET_CANCONTINUE()       \
    {                                           \
        OS_ClearReadySetCanContinue();          \
        _PIC18_ERRATA_NOP();                    \
    }
//------------------------------------------------------------------------------
#define _OS_RETURN_NO_SAVE()                    \
    {                                           \
        OS_SchedRetPointNoSave();               \
        _PIC18_ERRATA_NOP();                    \
    }                                           \
//------------------------------------------------------------------------------











/************************************************************************************************
 *                                                                                              *
 *     Create, replace, reserve task specific services                                          *
 *                                                                                              *
 ************************************************************************************************/


/************************************************************************/
/*                                                                      */
/* Replace current task with new (current task will be deleted)         */
/*                                                                      */
/************************************************************************/

#define OS_Task_Replace(priority, TaskName)                                     \
{                                                                               \
    *((NEAR _OST_INT_TYPE*)&OS_state) = priority | 0x48; /* bEnable  bReady */ \
    _OS_SET_FSR_CUR_TASK();                                                     \
    _postinc = priority | 0x48; /* bEnable  bReady */                          \
    _OS_SetIndfWord(&TaskName);                                                 \
    _OS_Return();                                                               \
}


/************************************************************************/
/*                                                                      */
/* Create task                                                          */
/*                                                                      */
/************************************************************************/

#define OS_Task_Create(priority, TaskName)                          \
    {                                                               \
        _OS_Task_Create(priority, (OST_CODE_POINTER)&(TaskName));   \
    }


/************************************************************************/
/*                                                                      */
/* Create task by indirrect address                                     */
/*                                                                      */
/************************************************************************/

// If task created indirrectly then it is nessesary to insert in
// the main():
// #pragma funcall main TaskName

#define OS_Task_Reserve(TaskName)       // *91210* Empty macro for this compiler



/************************************************************************************************
 *                                                                                              *
 *                                     I N T E R R U P T S                                      *
 *                                                                                              *
 ************************************************************************************************/

extern char OS_DI (void);
extern void OS_RI (char);
#define OS_EI()         { _gie = 1; if (_ipen) _giel = 1;}
#define OS_EIH()        { _gie = 1; }
#define OS_EIL()        { _giel = 1; }

//------------------------------------------------------------------------------
#if defined(OS_ENABLE_INT)
//------------------------------------------------------------------------------
    extern void OS_DI_INT (void);
    extern void OS_RI_INT (void);
//------------------------------------------------------------------------------
#endif
//------------------------------------------------------------------------------


/************************************************************************************************
 *                                                                                              *
 *                                     C O N D I T I O N S                                      *
 *                                                                                              *
 ************************************************************************************************/

extern void _OS_CheckEvent (_OST_INT_TYPE);

#define _OS_CHECK_EVENT(event) _OS_CheckEvent(event)




/******************************************************************************************
 *
 *  WORK WITH DYNAMIC TIMERS
 *
 ******************************************************************************************/

//------------------------------------------------------------------------------
#ifdef OS_ENABLE_DTIMERS
//------------------------------------------------------------------------------

#define _OS_IndfTimerTimeout       ((NEAR OST_DTIMER_FLAGS*)(&_indf))->bTimeout
#define _OS_IndfTimerActive        ((NEAR OST_DTIMER_FLAGS*)(&_indf))->bActive
#define _OS_IndfTimerRun           ((NEAR OST_DTIMER_FLAGS*)(&_indf))->bRun
#define _OS_IndfTimerNextEnable    ((NEAR OST_DTIMER_FLAGS*)(&_indf))->bNextEnable
#define _OS_PostincTimerNextEnable ((NEAR OST_DTIMER_FLAGS*)(&_postinc))->bNextEnable

//------------------------------------------------------------------------------
#endif  // OS_ENABLE_DTIMERS
//------------------------------------------------------------------------------



#endif





