/*
 ***********************************************************************************************
 *
 *  OSA cooperative RTOS for Microchip PIC-controllers: PIC10/12/16/18/24/dsPIC
 *
 *  OSA is distributed under BSD license (see license.txt)
 *
 *  URL:        http://wiki.pic24.ru/doku.php/en/osa/ref/intro
 *              http://picosa.narod.ru
 *
 *----------------------------------------------------------------------------------------------
 *
 *  File:       osa_bsem.h
 *
 *  Programmer: Timofeev Victor
 *              osa@pic24.ru, testerplus@mail.ru
 *
 *  Definition: Services for work with binary semaphores
 *
 *  History:    21.01.2009
 * 
 *              10.12.2009      1. Services for interrupt were corrected.
 *                                  Modified:
 *                                      OS_Bsem_Wait
 *                                      OS_Bsem_Wait_TO
 *                                  Added:
 *                                      OS_Bsem_Check_I
 *                                      OS_Bsem_Reset_I
 *                                      OS_Bsem_Set_I
 *                                      OS_Bsem_Switch_I
 *                                      OS_Bsem_Signal_I
 * 
 *
 ************************************************************************************************
 */

/************************************************************************************************
 *                                                                                              *
 *                          B I N A R Y    S E M A P H O R E S                                  *
 *                                                                                              *
 ************************************************************************************************/


#if defined(OS_ENABLE_INT_BSEM)

    #define __OS_BSEM_DI()      OS_DI_INT()
    #define __OS_BSEM_RI()      OS_RI_INT()

#else

    #define __OS_BSEM_DI()
    #define __OS_BSEM_RI()

#endif




#define _OS_BSEM_POS(binsem)        ((binsem) >> _OST_INT_SHIFT)
#define _OS_BSEM_MASK(binsem)       (1 << ((binsem) & _OST_INT_MASK))




// Signal binary semaphore
//
#define OS_Bsem_Set(binsem)     OS_BSem[_OS_BSEM_POS(binsem)] |= _OS_BSEM_MASK(binsem)
#define OS_Bsem_Signal(binsem)  OS_Bsem_Set(binsem)



// Reset binary semaphore

#define OS_Bsem_Reset(binsem)   OS_BSem[_OS_BSEM_POS(binsem)] &= ~_OS_BSEM_MASK(binsem)



// Check for binary semaphore is set

#define OS_Bsem_Check(binsem)   (OS_BSem[_OS_BSEM_POS(binsem)] & _OS_BSEM_MASK(binsem))



// Inverce semaphore's value

#define OS_Bsem_Switch(binsem)  OS_BSem[_OS_BSEM_POS(binsem)] ^= _OS_BSEM_MASK(binsem)



//------------------------------------------------------------------------------
#if defined(OS_ENABLE_INT_BSEM)
//------------------------------------------------------------------------------

    // Wait binary semaphore

    #define OS_Bsem_Wait(binsem)                                            \
        OSM_BEGIN {                                                         \
            for (;;) {                                                      \
                _OS_WAIT_EVENT(OS_Bsem_Check(binsem));                      \
                __OS_BSEM_DI();                                             \
                if (OS_Bsem_Check(binsem)) break;                           \
                __OS_BSEM_DI();                                             \
            }                                                               \
            OS_Bsem_Reset(binsem);                                          \
            __OS_BSEM_RI();                                                 \
        } OSM_END



    // Wait binary semaphore. Exit if timeout expired

    #define OS_Bsem_Wait_TO(binsem, timeout)                                \
        OSM_BEGIN {                                                         \
            for (;;) {                                                      \
                _OS_WAIT_EVENT_TIMEOUT(OS_Bsem_Check(binsem), timeout);     \
                __OS_BSEM_DI();                                             \
                if (OS_Bsem_Check(binsem) || OS_IsTimeout()) break;         \
                __OS_BSEM_RI();                                             \
            }                                                               \
            OS_Bsem_Reset(binsem);                                          \
            __OS_BSEM_RI();                                                 \
        } OSM_END


    #define OS_Bsem_Set_I(binsem)           OS_Bsem_Set(binsem)
    #define OS_Bsem_Reset_I(binsem)         OS_Bsem_Reset(binsem)
    #define OS_Bsem_Check_I(binsem)         OS_Bsem_Check(binsem)
    #define OS_Bsem_Signal_I(binsem)        OS_Bsem_Signal(binsem)
    #define OS_Bsem_Switch_I(binsem)        OS_Bsem_Switch(binsem)


//------------------------------------------------------------------------------
#else   // OS_ENABLE_INT_BSEM
//------------------------------------------------------------------------------

    // Wait binary semaphore

    #define OS_Bsem_Wait(binsem)                                            \
        OSM_BEGIN {                                                         \
            _OS_WAIT_EVENT(OS_Bsem_Check(binsem));                          \
            OS_Bsem_Reset(binsem);                                          \
        } OSM_END



    // Wait binary semaphore. Exit if timeout expired

    #define OS_Bsem_Wait_TO(binsem, timeout)                                \
        OSM_BEGIN {                                                         \
            _OS_WAIT_EVENT_TIMEOUT(OS_Bsem_Check(binsem), timeout);         \
            OS_Bsem_Reset(binsem);                                          \
        } OSM_END

//------------------------------------------------------------------------------
#endif  // OS_ENABLE_INT_BSEM
//------------------------------------------------------------------------------




















